!********************************************************************************
!** CONSTANT-NPH MOLECULAR DYNAMICS - EXTENDED SYSTEM METHOD                   **
!   (Modified by Yoshi, Jan 2005, from Allen-Tildesley's F28)
!********************************************************************************

!C    *******************************************************************
!C    ** CONSTANT-NVT MOLECULAR DYNAMICS USING AN EXTENDED SYSTEM.     **
!C    **                                                               **
!C    ** REFERENCE:                                                    **
!C    **                                                               **
!C    ** NOSE, MOLEC. PHYS. 52, 255, 1984.                             **
!C    **                                                               **
!C    ** ROUTINES:                                                     **
!C    **                                                               **
!C    ** SUBROUTINE PREDIC                                             **
!C    **    PREDICTS POSITIONS AT TIME T + DT                          **
!C    ** SUBROUTINE CORREC                                             **
!C    **    CORRECTS POSITIONS AT TIME T + DT                          **
!C    **                                                               **
!C    ** PRINCIPAL VARIABLES:                                          **
!C    **                                                               **
!C    ** INTEGER N                             NUMBER OF ATOMS         **
!C    ** REAL    DT                            TIMESTEP                **
!C    ** REAL    RX(N),RY(N),RZ(N)             ATOM POSITIONS          **
!C    ** REAL    VX(N),VY(N),VZ(N)             FIRST DERIVATIVES       **
!C    ** REAL    AX(N),AY(N),AZ(N)             SECOND DERIVATIVES      **
!C    ** REAL    BX(N),BY(N),BZ(N)             THIRD DERIVATIVES       **
!C    ** REAL    FX(N),FY(N),FZ(N)             TOTAL FORCES            **
!C    ** REAL    V,VV,VA,VB,VC,VF              S AND DERIVATIVES       **
!C    ** REAL    WW                            MASS OF PISTON          **
!C    **                                                               **
!C    ** USAGE:                                                        **
!C    **                                                               **
!C    ** THE MODIFIED EQUATIONS OF MOTION ARE AS FOLLOWS:              **
!C    ** A  = F/WM + (R/3)[AV/V - (2/3)*(VV/V)**2]                     **
!C    ** VA = ( PRES - PRESUR ) / WW                                   **
!C    **   PRES = (1/3V) * [ WM * SUM{(V-R*(VV/3V))**2} + VIRIAL       **
!C    **   VIRIAL = SUM{R dot F}                                       **
!C    ** WHERE R,V,A,.. ARE SUCCESSIVE DERIVATIVES OF POSITION         **
!C    ** AND VV,VA,.... ARE SUCCESSIVE DERIVATIVES OF VOLUME           **
!C    ** WE SOLVE THESE EQUATIONS BY A GEAR 5-VALUE METHOD FOR SECOND  **
!C    ** ORDER DIFFERENTIAL EQUATIONS.                                 **
!C    ** THIS PROGRAM USES UNIT 10 FOR CONFIGURATION INPUT AND OUTPUT  **
!C    *******************************************************************

!C    *******************************************************************
!C    ** STANDARD TAYLOR SERIES PREDICTORS                             **
!C    *******************************************************************

        SUBROUTINE gear_pc_nph_predic
        use variables
        use constants
        implicit double precision (a-h,o-z)

        INTEGER :: I
        double precision :: C1, C2, C3, C4

!C    *******************************************************************

        C1 = DT
        C2 = C1 * DT / 2.0
        C3 = C2 * DT / 3.0
        C4 = C3 * DT / 4.0

        DO 100 I = 1, N

           RX(I) = RX(I) + C1*VX(I) + C2*AX(I) + C3*BX(I) + C4*CX(I)
           RY(I) = RY(I) + C1*VY(I) + C2*AY(I) + C3*BY(I) + C4*CY(I)
           RZ(I) = RZ(I) + C1*VZ(I) + C2*AZ(I) + C3*BZ(I) + C4*CZ(I)
           VX(I) = VX(I) + C1*AX(I) + C2*BX(I) + C3*CX(I)
           VY(I) = VY(I) + C1*AY(I) + C2*BY(I) + C3*CY(I)
           VZ(I) = VZ(I) + C1*AZ(I) + C2*BZ(I) + C3*CZ(I)
           AX(I) = AX(I) + C1*BX(I) + C2*CX(I)
           AY(I) = AY(I) + C1*BY(I) + C2*CY(I)
           AZ(I) = AZ(I) + C1*BZ(I) + C2*CZ(I)
           BX(I) = BX(I) + C1*CX(I)
           BY(I) = BY(I) + C1*CY(I)
           BZ(I) = BZ(I) + C1*CZ(I)

100     CONTINUE

        V  = V  + C1*VV + C2*VA + C3*VB + C4*VC
        VV = VV + C1*VA + C2*VB + C3*VC
        VA = VA + C1*VB + C2*VC
        VB = VB + C1*VC

        RETURN
        END

!C    *******************************************************************
!C    ** GEAR CORRECTOR ALGORITHM                                      **
!C    **                                                               **
!C    ** WE USE GEAR'S ALTERNATIVE COEFFICIENT GEAR0 WHICH APPLIES IN  **
!C    ** THE CASE THAT FIRST TIME DERIVATIVES APPEAR ON THE RIGHT OF   **
!C    ** THESE SECOND-ORDER DIFFERENTIAL EQUATIONS.                    **
!C    ** FOR TIMESTEP-SCALED VARIABLES THE COEFFICIENTS WOULD BE       **
!C    ** 19/90, 3/4, 1, 1/2, 1/12.                                     **
!C    **                                                               **
!C    ** REFERENCES:                                                   **
!C    **                                                               **
!C    ** GEAR, NUMERICAL INITIAL VALUE PROBLEMS IN ORDINARY            **
!C    ** DIFFERENTIAL EQUATIONS (PRENTICE-HALL, 1971).                 **
!C    ** GEAR, REPORT ANL 7126 (ARGONNE NATIONAL LAB., 1966).          **
!C    *******************************************************************

        SUBROUTINE gear_pc_nph_correc
        use variables
        use constants
        implicit double precision (a-h,o-z)

        double precision :: GEAR0, GEAR1, GEAR3, GEAR4
        PARAMETER ( GEAR0 = 19.0 / 90.0, GEAR1 = 3.0 /4.0, &
        GEAR3 = 1.0 / 2.0,   GEAR4 = 1.0 /12.0 )

!C    *******************************************************************

!  Kinetic energy has to be calculated using P. Not V.
        enkin = 0.0; enkineach=0.0
        V1V3 = VV / V / 3.0D0
        DO I = 1, N
           XX = VX(I) - RX(I) * V1V3
           YY = VY(I) - RY(I) * V1V3
           ZZ = VZ(I) - RZ(I) * V1V3
           enkineach(i) = 0.5d0 * wm(i) * (XX ** 2 + YY ** 2 + ZZ ** 2)
           enkin = enkin + enkineach(i)
        ENDDO

        VF = -PEX/WW + 1.0D0/(3.0D0*V*WW)*(ENKIN*2.0D0+VIRX+VIRY+VIRZ)

        C1 = DT
        C2 = C1 * DT / 2.0
        C3 = C2 * DT / 3.0
        C4 = C3 * DT / 4.0

        CR = GEAR0 * C2
        CV = GEAR1 * C2 / C1
        CB = GEAR3 * C2 / C3
        CC = GEAR4 * C2 / C4

        VFAC = VA / (3.0D0*V) - (2.0D0/9.0D0)*(VV/V)**2

        DO 400 I = 1, N

           AXI = FX(I) / WM(I) + RX(I) * VFAC
           AYI = FY(I) / WM(I) + RY(I) * VFAC
           AZI = FZ(I) / WM(I) + RZ(I) * VFAC
           CORRX = AXI - AX(I)
           CORRY = AYI - AY(I)
           CORRZ = AZI - AZ(I)

           RX(I) = RX(I) + CR * CORRX
           RY(I) = RY(I) + CR * CORRY
           RZ(I) = RZ(I) + CR * CORRZ
           VX(I) = VX(I) + CV * CORRX
           VY(I) = VY(I) + CV * CORRY
           VZ(I) = VZ(I) + CV * CORRZ
           AX(I) = AXI
           AY(I) = AYI
           AZ(I) = AZI
           BX(I) = BX(I) + CB * CORRX
           BY(I) = BY(I) + CB * CORRY
           BZ(I) = BZ(I) + CB * CORRZ
           CX(I) = CX(I) + CC * CORRX
           CY(I) = CY(I) + CC * CORRY
           CZ(I) = CZ(I) + CC * CORRZ

400     CONTINUE

        CORR = VF - VA
        V  = V  + CR * CORR
        VV = VV + CV * CORR
        VA = VF
        VB = VB + CB * CORR
        VC = VC + CC * CORR

        vpth=v**(1.0d0/3.0d0)
        hmat(1,1)=vpth
        hmat(2,2)=vpth
        hmat(3,3)=vpth

        RETURN
        END


